# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Network search and information tools

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from mywigle.http import get
from mywigle.http import post


def comment(netid, comment):
    """Add a comment to a network

    Provide custom information regarding a single network.

    Parameters
    ----------
    netid : str
        The BSSID of the network for the comment, e.g. '0A:2C:EF:3D:25:1B'.
    comment : str
        The comment to attach.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return post("network", "comment", **locals())


def detail(
    netid=None,
    operator=None,
    lac=None,
    cid=None,
    system=None,
    network=None,
    basestation=None,
):
    """Get details and observation records for a single network

    Provide unique information for a WiFi or cell network to request detailed
    information. Providing a netId value searches WiFi, operator searches GSM,
    and system searches CDMA.

    Parameters
    ----------
    netid : str, optional
        The WiFi Network BSSID to search.
    operator : int, optional
        GSM Operator ID.
    lac : int, optional
        GSM Location Area Code.
    cid : long, optional
        GSM Cell ID.
    system : int, optional
        CDMA System ID.
    network : int, optional
        CDMA Network ID.
    basestation : int, optional
        CDMA Base Station ID.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("network", "detail", **locals())


def geocode(addresscode=None):
    """Get coordinates for an address for use in searching

    Relies on OpenStreetMap nominatim.

    Parameters
    ----------
    addresscode : str
        An address string, Street, City, State/Region, Country.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("network", "geocode", **locals())


def search(
    onlymine=None,
    first=None,
    latrange1=None,
    latrange2=None,
    longrange1=None,
    longrange2=None,
    lastupdt=None,
    freenet=None,
    paynet=None,
    netid=None,
    ssid=None,
    ssidlike=None,
    variance=None,
    resultsPerPage=None,
):
    """Search the WiGLE database

    Query the WiGLE database for paginated results based on multiple criteria.
    API and session authentication default to a page size of 100 results/page.
    COMMAPI defaults to a page size of 25 with a maximum of 1000 results per
    return. Number of daily queries allowed per user are throttled based on
    history and participation

    Parameters
    ----------
    onlymine : boolean, optional
        Search only for points first discovered by the current user. Use any
        string to set, leave unset for general search. Can't be used with
        COMMAPI auth, since these are points you have locally.
    first : int, optional
        Result offset to fetch, used to page through results. Defaults to 0.
        COMMAPI queries are bounded at 100 pages. If you need more open-ended
        searches than this, we recommend contacting WiGLE-admin@wigle.net to
        discuss bulk pricing.
    latrange1 : float, optional
        Lesser of two latitudes by which to bound the search.
    latrange2 : float, optional
        Greater of two latitudes by which to bound the search.
    longrange1 : float, optional
        Lesser of two latitudes by which to bound the search.
    longrange2 : float, optional
        Greater of two longitudes by which to bound the search.
    lastupdt : str, optional
        Filter points by how recently they've been updated, condensed date/time
        numeric string format 'yyyyMMddhhmmss'
    freenet : boolean, optional
        Include only networks that have been marked as free access.
    paynet : boolean, optional
        Include only networks that have been marked as for-pay access.
    netid : str, optional
        Include only networks matching the string network BSSID, e.g.
        '0A:2C:EF:3D:25:1B' or '0A:2C:EF'. The first three octets are required.
    ssid : str, optional
        Include only networks exactly matching the string network name.
    ssidlike : str, optional
        Include only networks matching the string network name, allowing
        wildcards '%' (any string) and '_' (any character).
    variance : float, optional
        How tightly to bound queries against the provided latitude/longitude
        box. Value must be between 0.001 and 0.2
    resultsPerPage : int, optional
        How many results to return per request. Defaults to 25 for COMMAPI, 10
        for site. Bounded at 1000 for COMMAPI, 100 for site.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("network", "search", **locals())



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Statistics and information

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from mywigle.http import get


def apiToken(token_type=None):
    """Get all authorization tokens for the logged-in user

    Parameters
    ----------
    token_type : str
        Token types - 'API', 'COMMAPI', or 'ANDROID'.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    params = {"type": token_type}  # required because "type" is a reserved word
    return get("profile", "apiToken", **params)


def user():
    """Get the user object for the current logged-in user

    See basic user information.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("profile", "user")



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

import os
import requests
from platform import system
import re
import subprocess
import time


class MyWigleClient:
    """
    Wrapper for Wigle API: search, statistic, uploads, profile, sniffer.
    """

    def __init__(self, user: str, key: str):
        self.user = user
        self.key = key
        self.url = "https://api.wigle.net/api/v2/{section}/{endpoint}"
        self.auth = requests.auth.HTTPBasicAuth(self.user, self.key)
        if self.user == "" or self.key == "":
            print(
                "Besuche https://wigle.net/account, um deinen API-Token zu holen und ihn hier einzutragen."
            )

    # ---- HTTP-Methods ----
    def _get(self, section, endpoint, **kwargs):
        path = self.url.format(section=section, endpoint=endpoint)
        r = requests.get(path, auth=self.auth, params=kwargs)
        r.raise_for_status()
        return r.json()

    def _post(self, section, endpoint, **kwargs):
        path = self.url.format(section=section, endpoint=endpoint)
        r = requests.post(path, auth=self.auth, params=kwargs)
        r.raise_for_status()
        return r.json()

    # ---- Network-Funktions ----
    def network_comment(self, netid, comment):
        return self._post("network", "comment", netid=netid, comment=comment)

    def network_detail(
        self,
        netid=None,
        operator=None,
        lac=None,
        cid=None,
        system=None,
        network=None,
        basestation=None,
    ):
        return self._get(
            "network",
            "detail",
            netid=netid,
            operator=operator,
            lac=lac,
            cid=cid,
            system=system,
            network=network,
            basestation=basestation,
        )

    def network_geocode(self, addresscode=None):
        return self._get("network", "geocode", addresscode=addresscode)

    def network_search(
        self,
        onlymine=None,
        first=None,
        latrange1=None,
        latrange2=None,
        longrange1=None,
        longrange2=None,
        lastupdt=None,
        freenet=None,
        paynet=None,
        netid=None,
        ssid=None,
        ssidlike=None,
        variance=None,
        resultsPerPage=None,
    ):
        return self._get(
            "network",
            "search",
            onlymine=onlymine,
            first=first,
            latrange1=latrange1,
            latrange2=latrange2,
            longrange1=longrange1,
            longrange2=longrange2,
            lastupdt=lastupdt,
            freenet=freenet,
            paynet=paynet,
            netid=netid,
            ssid=ssid,
            ssidlike=ssidlike,
            variance=variance,
            resultsPerPage=resultsPerPage,
        )

    # ---- statistics ----
    def stats_countries(self):
        return self._get("stats", "countries")

    def stats_general(self):
        return self._get("stats", "general")

    def stats_group(self):
        return self._get("stats", "group")

    def stats_regions(self, country=None):
        return self._get("stats", "regions", country=country)

    def stats_site(self):
        return self._get("stats", "site")

    def stats_standings(self, sort=None, pagestart=None, pageend=None):
        return self._get(
            "stats", "standings", sort=sort, pagestart=pagestart, pageend=pageend
        )

    def stats_user(self):
        return self._get("stats", "user")

    # ---- FileUploads ----
    def file_kml(self, transid):
        endpoint = f"kml/{transid}"
        return self._get("file", endpoint)

    def file_transactions(self, pagestart=None, pageend=None):
        return self._get("file", "transactions", pagestart=pagestart, pageend=pageend)

    def file_upload(self, file, donate=None):
        # file must be a File-Objekt
        files = {"file": file}
        params = {}
        if donate is not None:
            params["donate"] = donate
        path = self.url.format(section="network", endpoint="upload")
        r = requests.post(path, auth=self.auth, files=files, data=params)
        r.raise_for_status()
        return r.json()

    # ---- Profile ----
    def profile_apiToken(self, token_type=None):
        params = {"type": token_type} if token_type else {}
        return self._get("profile", "apiToken", **params)

    def profile_user(self):
        return self._get("profile", "user")

    # ---- Sniffer (local WiFi-Analysis) ----
    def sniffer_local_bssids(self):
        scan_cmd = {
            "windows": "netsh wlan show networks mode=Bssid",
            "linux": "nmcli -t -f bssid dev wifi list",
            "darwin": "/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport -s",
        }
        sysname = system().lower()
        cmd = scan_cmd.get(sysname)
        if not cmd:
            raise Exception("system not supported for WiFi scan!")
        res = subprocess.check_output(cmd, shell=True, universal_newlines=True)
        pattern = re.compile(r"(?:[0-9a-fA-F]:?){12}")
        bssids = pattern.findall(res)
        return set(bssids)

    def sniffer_geolocate(self, geocoder="wigle"):
        lats = []
        longs = []
        for bssid in self.sniffer_local_bssids():
            print("BSSID:", bssid)
            if geocoder == "wigle":
                lat, lng = self.sniffer_geolocate_wigle(bssid)
            elif geocoder == "google":
                lat, lng = self.sniffer_geolocate_google(bssid)
            else:
                lat, lng = None, None
            print(lat, lng)
            if lat:
                lats.append(lat)
            if lng:
                longs.append(lng)
            time.sleep(0.1)
        if lats:
            lat = sum(lats) / len(lats)
            lng = sum(longs) / len(longs)
            return lat, lng
        else:
            return "No Geolocalisation possible"

    def sniffer_geolocate_wigle(self, bssid):
        res = self.network_search(netid=bssid)
        if res.get("success") and res.get("resultCount"):
            lat = res["results"][0]["trilat"]
            lng = res["results"][0]["trilong"]
        else:
            print(res)
            lat, lng = None, None
        return lat, lng

    def sniffer_geolocate_google(self, bssid):
        geolocate_cmd = "https://maps.googleapis.com/maps/api/browserlocation/json?browser=firefox&wifi=mac:{bssid}"
        res = requests.get(geolocate_cmd.format(bssid=bssid))
        res.raise_for_status()
        res = res.json()
        if res.get("status") == "OK":
            lat = res["location"]["lat"]
            lng = res["location"]["lng"]
        else:
            lat, lng = None, None
        return lat, lng



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Configuration for the Wigle API.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os


url = "https://api.wigle.net/api/v2/{section}/{endpoint}"
# Visit https://wigle.net/account to collect your API token
user = ""
key = ""

try:
    # for use in dev environment
    from mywigle.dev.config import *

    print(user)
    print(key)
except ImportError:
    pass

if user == "" or key == "":
    print("Visit https://wigle.net/account to collect your API token")
    here = os.path.abspath(__file__)
    here.replace(".pyc", ".py")
    print("Then add your credentials to %s" % here)



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Network observation file upload and status

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from mywigle.http import get
from mywigle.http import post


def kml(transid):
    """Download a KML summary of a file

    Get a KML summary approximation for a successfully processed file uploaded
    by the current user.

    Parameters
    ----------
    transid : str
        The unique transaction ID for the file.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("file", "kml/{transid}".format(**locals()))


def transactions(pagestart=None, pageend=None):
    """Get the status of files uploaded by the current user

    Paginated at 50 results per page.

    Parameters
    ----------
    pagestart : int, optional
        Most recent record to fetch descending chronologically. Defaults to 0.
    pageend : int, optional
        Number of results to fetch from offset. Defaults to 100.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("file", "transactions", **locals())


def upload(file, donate=None):
    """Upload a file

    Transmit a file for processing and incorporation into the database.

    Parameters
    ----------
    file : file or file-like object
        Supports DStumbler, G-Mon, inSSIDer, Kismac, Kismet, MacStumbler,
        NetStumbler, Pocket Warrior, Wardrive-Android, WiFiFoFum, WiFi-Where,
        WiGLE WiFi Wardriving, and Apple consolidated DB formats. One or more
        files may be enclosed within a zip, tar, or tar.gz archive. Files may
        not exceed 140MiB, and archives WILL IGNORE more than 200 member files.
    donate : boolean, optional
        Allow commercial use of the file contents - True to allow.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return post("network", "upload", **locals())



# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Sniff local WiFi networks in order to look them up on WiGLE.

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from platform import system
import re
import subprocess
import time

import requests

from mywigle import network


scan_cmd = {
    "windows": "netsh wlan show networks mode=Bssid",
    "linux": "nmcli -t -f bssid dev wifi list",
    "osx": "/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport -s",
}
geolocate_cmd = {
    "google": "https://maps.googleapis.com/maps/api/browserlocation/json?browser=firefox&wifi=mac:{bssid}"
}


def local_bssids():
    res = subprocess.check_output(scan_cmd[system().lower()], universal_newlines=True)
    pattern = re.compile(r"(?:[0-9a-fA-F]:?){12}")
    bssids = pattern.findall(res)
    return set(bssids)


def geolocate(geocoder):
    lats = []
    longs = []
    for bssid in local_bssids():
        print("BSSID:", bssid)
        if geocoder == "wigle":
            lat, lng = geolocate_wigle(bssid)
        elif geocoder == "google":
            lat, lng = geolocate_google(bssid)
        print(lat, lng)
        if lat:
            lats.append(lat)
        if lng:
            longs.append(lng)
        time.sleep(0.1)
    if lats:
        lat = sum(lats) / len(lats)
        lng = sum(longs) / len(longs)
        return lat, lng
    else:
        return "No geolocation possible"


def geolocate_wigle(bssid):
    res = network.search(netid=bssid)
    if res["success"] and res["resultCount"]:
        lat = res["results"][0]["trilat"]
        lng = res["results"][0]["trilong"]
    else:
        print(res)
        lat, lng = None, None
    return lat, lng


def geolocate_google(bssid):
    res = requests.get(geolocate_cmd["google"], params={"mac": bssid})
    res.raise_for_status()
    res = res.json()
    if res["status"] == "OK":
        lat = res["location"]["lat"]
        lng = res["location"]["lng"]
    else:
        lat, lng = None, None
    return lat, lng


if __name__ == "__main__":
    print("WiGLE")
    print(geolocate("wigle"))
    print("Google")
    print(geolocate("google"))



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Statistics and information

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from mywigle.http import get


def countries():
    """Get statistics organized by country

    Fetches all countries and basic stats.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "countries")


def general():
    """Get a named map of general statistics

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "general")


def group():
    """Get group standings

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "group")


def regions(country=None):
    """Get statistics for a specified country, organized by region

    Parameters
    ----------
    country : str, optional
        The two-letter code of the country for which you'd like a regional
        breakdown. Defaults to 'US'

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "regions", **locals())


def site():
    """Get a named map of site-level statistics

    A big hash of short-named statistics used in providing site-wide
    information.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "site")


def standings(sort=None, pagestart=None, pageend=None):
    """Get user standings

    Parameters
    ----------
    sort : str
        The criteria by which to sort the results. Values are ['discovered',
        'total', 'monthcount', 'prevmonthcount', 'gendisc', 'gentotal',
        'firsttransid', 'lasttransid'].
    pagestart : int
        The first record to request according to the 'sort' parameter.
    pageend : int
        The last record to request according to the 'sort' parameter.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "standings", **locals())


def user():
    """Get user statistics

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    return get("stats", "user")



# -*- coding: iso-8859-1 -*-
# Copyright (c) 2017 Jamie Bull - oCo Carbon Ltd
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
API wrapper for Wigle, WiFi geolocation.

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import requests

from mywigle.config import key
from mywigle.config import url
from mywigle.config import user


auth = requests.auth.HTTPBasicAuth(user, key)


def get(section, endpoint, **kwargs):
    """Make a GET request using Basic auth.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    path = url.format(**locals())
    r = requests.get(path, auth=auth, params=kwargs)
    r.raise_for_status()
    return r.json()


def post(section, endpoint, **kwargs):
    """Make a POST request using Basic auth.

    Returns
    -------
    dict

    Raises
    ------
    requests.HTTPError

    """
    r = requests.post(url.format(**locals()), auth=auth, params=kwargs)
    r.raise_for_status()
    return r.json()
